"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var cross_fetch_1 = require("cross-fetch");
var Promise = require("promise");
var flowrunner_1 = require("@devhelpr/flowrunner");
var getPayload = function (node, data) {
    var _a, _b;
    var propertyName = node.propertyName || 'data';
    var payload;
    if (!!node.insertResultDirectIntoPayload) {
        if (node.useChildProperty) {
            payload = Object.assign({}, node.payload, __assign({}, data[node.useChildProperty]));
        }
        else {
            payload = Object.assign({}, node.payload, __assign({}, data));
        }
    }
    else if (node.useChildProperty) {
        payload = Object.assign({}, node.payload, (_a = {}, _a[propertyName] = data[node.useChildProperty], _a));
    }
    else {
        payload = Object.assign({}, node.payload, (_b = {}, _b[propertyName] = data, _b));
    }
    return payload;
};
var FetchTask = /** @class */ (function (_super) {
    __extends(FetchTask, _super);
    function FetchTask() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.replacePropertiesInUrl = function (url, data) {
            // parse {} from url and replace with predefined variabels
            var regularExpressionForMatchingCurlyBrackets = /{([^}]+)}/g, currentMatch;
            while ((currentMatch = regularExpressionForMatchingCurlyBrackets.exec(url))) {
                var value = '';
                var field = currentMatch[1];
                if (data[field]) {
                    value = data[field];
                }
                else {
                    value = '';
                }
                url = url.replace('{' + currentMatch[1] + '}', value);
            }
            return url;
        };
        return _this;
    }
    FetchTask.prototype.execute = function (node, services) {
        var cleanPayload = Object.assign({}, node.payload);
        cleanPayload.response = undefined;
        cleanPayload.request = undefined;
        var url = this.replacePropertiesInUrl(node.url, cleanPayload);
        return new Promise(function (resolve, reject) {
            if (url != '') {
                if (node.method === undefined || node.method == '' || node.method == 'get') {
                    cross_fetch_1.default(url, {
                        headers: {
                            'Cache-Control': 'no-cache',
                        },
                    })
                        .then(function (res) {
                        if (res.status >= 400) {
                            throw new Error(res.status.toString());
                        }
                        return res.json();
                    })
                        .then(function (data) {
                        resolve(getPayload(node, data));
                    })
                        .catch(function (err) {
                        console.error(err);
                        node.payload = Object.assign({}, node.payload, {
                            followFlow: 'isError',
                        });
                        resolve(node.payload);
                    });
                }
                else {
                    cross_fetch_1.default(url, {
                        method: node.method.toUpperCase(),
                        body: JSON.stringify(cleanPayload),
                        headers: {
                            'Content-Type': 'application/json',
                        },
                    })
                        .then(function (res) {
                        if (res.status >= 400) {
                            throw new Error(res.status.toString());
                        }
                        return res.json();
                    })
                        .then(function (data) {
                        resolve(getPayload(node, data));
                    })
                        .catch(function (err) {
                        console.error(node.method, err);
                        node.payload = Object.assign({}, node.payload, {
                            followFlow: 'isError',
                        });
                        resolve(node.payload);
                    });
                }
            }
            else {
                reject();
            }
        });
    };
    FetchTask.prototype.getDescription = function () {
        return 'Node that fetches data from an url';
    };
    FetchTask.prototype.getName = function () {
        return 'FetchTask';
    };
    FetchTask.prototype.getFullName = function () {
        return 'Fetch';
    };
    FetchTask.prototype.getIcon = function () {
        return 'fetch';
    };
    FetchTask.prototype.getShape = function () {
        return 'rect';
    };
    FetchTask.prototype.getDefaultColor = function () {
        return '#00ff80ff';
    };
    FetchTask.prototype.getTaskType = function () {
        return 'both';
    };
    FetchTask.prototype.getPackageType = function () {
        return flowrunner_1.FlowTaskPackageType.DEFAULT_NODE;
    };
    FetchTask.prototype.getCategory = function () {
        return 'FlowCanvas';
    };
    FetchTask.prototype.getController = function () {
        return 'FlowCanvasController';
    };
    FetchTask.prototype.getConfigMetaData = function () {
        return [
            { name: 'url', defaultValue: '', valueType: 'string', required: true },
            {
                name: 'method',
                defaultValue: 'get',
                valueType: 'enum',
                enumValues: ['get', 'post', 'delete', 'put'],
                enumText: ['get', 'post', 'delete', 'put'],
            },
        ];
    };
    return FetchTask;
}(flowrunner_1.FlowTask));
exports.FetchTask = FetchTask;
